@@ -3,8 +3,9 @@ from typing import Any, Sequence, cast, get_args
 
 from msgspec import Struct
 from msgspec.json import schema_components
+from pydantic import BaseModel
 
-from lihil.config import OASConfig
+from lihil.config.app_config import IOASConfig
 from lihil.constant.status import phrase
 from lihil.interface import RegularTypes, is_provided, is_set
 from lihil.oas import model as oasmodel
@@ -17,12 +18,11 @@ from lihil.problems import (
 from lihil.routing import Endpoint, Route, RouteBase
 from lihil.signature import EndpointSignature, RequestParam
 from lihil.utils.string import to_kebab_case, trimdoc
+from lihil.utils.typing import lenient_issubclass
 
-# from lihil.utils.json import encode_json
-
-type SchemasDict = dict[str, oasmodel.LenientSchema]
-type SecurityDict = dict[str, oasmodel.SecurityScheme | oasmodel.Reference]
-type ComponentsDict = dict[str, Any]
+SchemasDict = dict[str, oasmodel.LenientSchema]
+SecurityDict = dict[str, oasmodel.SecurityScheme | oasmodel.Reference]
+ComponentsDict = dict[str, Any]
 
 
 class DefinitionOutput(Struct):
@@ -35,9 +35,11 @@ class ReferenceOutput(Struct):
     component: SchemasDict
 
 
-type SchemaOutput = DefinitionOutput | ReferenceOutput
+SchemaOutput = DefinitionOutput | ReferenceOutput
 """When component is not None result contains reference"""
 
+MSGSPEC_REF_TEMPLATE = "#/components/schemas/{name}"
+PYDANTIC_REF_TEMPLATE = "#/components/schemas/{model}"
 PROBLEM_CONTENTTYPE = "application/problem+json"
 
 
@@ -45,10 +47,38 @@ class OneOfOutput(Struct):
     oneOf: list[SchemaOutput]
 
 
+def pydantic_json_schema(t: type[BaseModel]) -> SchemaOutput:
+    if not lenient_issubclass(t, BaseModel):
+        raise NotImplementedError
+
+    # Full schema including $defs and root schema
+    full_schema = t.model_json_schema(ref_template=PYDANTIC_REF_TEMPLATE)
+
+    # Extract components ($defs)
+    defs = full_schema.pop("$defs", {})
+
+    # The root schema describes `t` itself
+    root_schema = full_schema
+
+    # Build component dictionary
+    comp_dict = {name: oasmodel.Schema(**schema) for name, schema in defs.items()}
+
+    # Add root schema under the model's name
+    comp_dict[t.__name__] = oasmodel.Schema(**root_schema)
+
+    # Reference to use in OpenAPI output
+    ref = oasmodel.Reference(ref=PYDANTIC_REF_TEMPLATE.format(model=t.__name__))
+
+    return ReferenceOutput(ref, cast(SchemasDict, comp_dict))
+
+
 def json_schema(types: RegularTypes) -> SchemaOutput:
+    if lenient_issubclass(types, BaseModel):
+        return pydantic_json_schema(types)
+
     (schema,), definitions = schema_components(
         (types,),
-        ref_template="#/components/schemas/{name}",
+        ref_template=MSGSPEC_REF_TEMPLATE,
     )
 
     if anyOf := schema.pop("anyOf", None):  # rename
@@ -120,7 +150,8 @@ def detail_base_to_content(
 
     example = err_type.__json_example__()
     # Add a link to the problems page for this error type
-    problem_link = f"/problems?search={example["type_"]}"
+    problem_type = example["type_"]
+    problem_link = f"/problems?search={problem_type}"
     schemas[err_name] = oasmodel.Schema(
         type="object",
         properties=properties,
@@ -140,12 +171,12 @@ def detail_base_to_content(
 
 
 def _single_field_schema(
-    param: RequestParam[Any], schemas: SchemasDict
+    param: "RequestParam[Any]", schemas: SchemasDict
 ) -> oasmodel.Parameter:
     output = json_schema(param.type_)
     param_schema: dict[str, Any] = {
         "name": param.alias,
-        "in_": param.location,
+        "in_": param.source,
         "required": True,
     }
     if output.component:  # reference
@@ -174,7 +205,8 @@ def example_from_detail_base(
     err_name = err_type.__name__
 
     # Create a schema for this specific error type
-    problem_url = f"{problem_path}/search?{example["type_"]}"
+    problem_type = example["type_"]
+    problem_url = f"{problem_path}/search?{problem_type}"
     error_schema = oasmodel.Schema(
         type="object",
         title=err_name,  # Add title to make it show up in Swagger UI
@@ -200,7 +232,7 @@ def body_schema(
     if not (body_param := ep_deps.body_param):
         return None
     _, param = body_param
-    content = type_to_content(param.type_, schemas)
+    content = type_to_content(param.type_, schemas, param.content_type)
     body = oasmodel.RequestBody(content=content, required=True)
     return body
 
@@ -420,7 +452,7 @@ class ValidationErrors(Struct):
 
 def generate_oas(
     routes: Sequence[RouteBase],
-    oas_config: OASConfig,
+    oas_config: IOASConfig,
     app_version: str,
 ) -> oasmodel.OpenAPI:
     "Return application/json response"
@@ -438,14 +470,14 @@ def generate_oas(
             route=route,
             schemas=schemas,
             security_schemas=security_schemas,
-            problem_path=oas_config.problem_path,
+            problem_path=oas_config.PROBLEM_PATH,
         )
 
     comp = oasmodel.Components(**components)
-    info = oasmodel.Info(title=oas_config.title, version=app_version)
+    info = oasmodel.Info(title=oas_config.TITLE, version=app_version)
 
     oas = oasmodel.OpenAPI(
-        openapi=oas_config.version,
+        openapi=oas_config.VERSION,
         info=info,
         paths=paths,
         components=comp,
