@@ -1,16 +1,17 @@
+from concurrent.futures.thread import ThreadPoolExecutor
 from functools import partial
-from inspect import isasyncgen, isgenerator
+from inspect import isasyncgen, iscoroutinefunction, isgenerator
 from types import MethodType
 from typing import (
     Any,
+    Awaitable,
     Callable,
+    Generic,
     Literal,
     Pattern,
-    Self,
     Sequence,
     TypedDict,
     Union,
-    Unpack,
     cast,
     overload,
 )
@@ -19,14 +20,12 @@ from ididi import Graph, INodeConfig
 from ididi.graph import Resolver
 from ididi.interfaces import IDependent
 from msgspec import field
-from starlette.requests import Request
-from starlette.responses import Response, StreamingResponse
+from starlette.responses import StreamingResponse
+from typing_extensions import Self, Unpack
 
 from lihil.asgi import ASGIBase
-from lihil.auth.oauth import AuthBase
-from lihil.config import AppConfig, lhl_get_config
 from lihil.constant.resp import METHOD_NOT_ALLOWED_RESP
-from lihil.errors import InvalidParamTypeError
+from lihil.ds.resp import StaticResponse
 from lihil.interface import (
     HTTP_METHODS,
     ASGIApp,
@@ -35,20 +34,24 @@ from lihil.interface import (
     IScope,
     ISend,
     MiddlewareFactory,
+    P,
+    R,
     Record,
+    T,
 )
-from lihil.plugins.bus import BusTerminal, Event, EventBus, MessageRegistry
-from lihil.problems import DetailBase, InvalidRequestErrors, get_solver
-from lihil.signature import EndpointSignature, ParseResult
+from lihil.plugins import IPlugin
+from lihil.plugins.auth.oauth import AuthBase
+from lihil.problems import DetailBase, get_solver
+from lihil.signature import EndpointParser, EndpointSignature, Injector, ParseResult
 from lihil.signature.returns import agen_encode_wrapper, syncgen_encode_wrapper
 from lihil.utils.string import (
     build_path_regex,
     generate_route_tag,
-    get_parent_path,
     merge_path,
     trim_path,
 )
 from lihil.utils.threading import async_wrapper
+from lihil.vendors import Request, Response
 
 
 class IEndpointProps(TypedDict, total=False):
@@ -64,6 +67,8 @@ class IEndpointProps(TypedDict, total=False):
     "Auth Scheme for access control"
     tags: Sequence[str] | None
     "OAS tag, endpoints with the same tag will be grouped together"
+    plugins: list[IPlugin]
+    "Decorators to decorate the endpoint function"
 
 
 class EndpointProps(Record, kw_only=True):
@@ -73,6 +78,7 @@ class EndpointProps(Record, kw_only=True):
     scoped: Literal[True] | None = None
     auth_scheme: AuthBase | None = None
     tags: Sequence[str] | None = None
+    plugins: list[IPlugin] = field(default_factory=list[IPlugin])
 
     @classmethod
     def from_unpack(cls, **iconfig: Unpack[IEndpointProps]):
@@ -86,24 +92,30 @@ class EndpointProps(Record, kw_only=True):
         return cls(**iconfig)  # type: ignore
 
 
-class Endpoint[R]:
+class Endpoint(Generic[R]):
     def __init__(
         self,
         route: "Route",
         method: HTTP_METHODS,
         func: Callable[..., R],
         props: EndpointProps,
+        workers: ThreadPoolExecutor | None,
     ):
         self._route = route
         self._method: HTTP_METHODS = method
         self._unwrapped_func = func
-        self._func = async_wrapper(func, threaded=props.to_thread)
+        self._func = async_wrapper(func, threaded=props.to_thread, workers=workers)
         self._props = props
         self._name = func.__name__
+        self.__is_setup: bool = False
 
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}({self._method}: {self._route.path!r} {self._func})"
 
+    @property
+    def route(self):
+        return self._route
+
     @property
     def props(self):
         return self._props
@@ -136,54 +148,42 @@ class Endpoint[R]:
     def unwrapped_func(self) -> Callable[..., R]:
         return self._unwrapped_func
 
-    def setup(self) -> None:
-        self._graph = self._route.graph
-        self._busterm = self._route.busterm
+    @property
+    def is_setup(self) -> bool:
+        return self.__is_setup
 
-        self._sig = EndpointSignature.from_function(
-            graph=self._route.graph,
-            route_path=self._route.path,
-            f=self._unwrapped_func,
-            app_config=self._route.app_config,
-        )
-
-        self._dep_items = self._sig.dependencies.items()
-        if self._sig.plugins:
-            self._plugin_items = self._sig.plugins.items()
-        else:
-            self._plugin_items = ()
-
-        self._static = self._sig.static
-
-        scoped_by_config = bool(self._props and self._props.scoped is True)
-
-        self._require_body: bool = self._sig.body_param is not None
-        self._status_code = self._sig.default_status
-        self._scoped: bool = self._sig.scoped or scoped_by_config
-        self._encoder = self._sig.return_encoder
-        self._media_type = next(iter(self._sig.return_params.values())).content_type
-
-    async def inject_plugins(
-        self, params: dict[str, Any], request: Request, resolver: Resolver
-    ):
-        for name, p in self._plugin_items:
-            ptype = p.type_
-            assert isinstance(ptype, type)
-
-            if issubclass(ptype, Request):
-                params[name] = request
-            elif issubclass(ptype, EventBus):
-                bus = self._busterm.create_event_bus(resolver)
-                params[name] = bus
-            elif issubclass(ptype, Resolver):
-                params[name] = resolver
-            elif p.processor:
-                await p.processor(params, request, resolver)
+    async def chainup_plugins(
+        self, func: Callable[..., Awaitable[R]], sig: EndpointSignature[R]
+    ) -> Callable[..., Awaitable[R]]:
+        seen: set[int] = set()
+        for decor in self._props.plugins:
+            if (decor_id := id(decor)) in seen:
+                continue
+            if iscoroutinefunction(decor):
+                wrapped = await decor(self._route.graph, func, sig)
             else:
-                raise InvalidParamTypeError(
-                    f"Plugin {p.plugin} has no processor defined"
-                )
-        return params
+                wrapped = decor(self._route.graph, func, sig)
+            func = cast(Callable[..., Awaitable[R]], wrapped)
+            seen.add(decor_id)
+        return func
+
+    async def setup(self, sig: EndpointSignature[R]) -> None:
+        if self.__is_setup:
+            raise Exception(f"`setup` is called more than once in {self}")
+
+        self._graph = self._route.graph
+        self._sig = sig
+        self._func = await self.chainup_plugins(self._func, self._sig)
+        self._injector = Injector(self._sig)
+
+        self._static = sig.static
+        self._status_code = sig.status_code
+        self._scoped: bool = sig.scoped or self._props.scoped is True
+        self._encoder = sig.encoder
+
+        self._media_type = sig.media_type
+
+        self.__is_setup = True
 
     async def make_static_call(
         self, scope: IScope, receive: IReceive, send: ISend
@@ -202,27 +202,9 @@ class Endpoint[R]:
         request = Request(scope, receive, send)
         callbacks = None
         try:
-            if self._require_body:
-                parsed_result = await self._sig.parse_command(request)
-            else:
-                parsed_result = self._sig.parse_query(request)
-
-            callbacks = parsed_result.callbacks
-            if errors := parsed_result.errors:
-                raise InvalidRequestErrors(detail=errors)
-
-            if self._plugin_items:
-                params = await self.inject_plugins(
-                    parsed_result.params, request, resolver
-                )
-            else:
-                params = parsed_result.params
-
-            for name, dep in self._dep_items:
-                params[name] = await resolver.aresolve(dep.dependent, **params)
-
-            raw_return = await self._func(**params)
-            return raw_return
+            parsed = await self._injector.validate_request(request, resolver)
+            params, callbacks = parsed.params, parsed.callbacks
+            return await self._func(**params)
         except Exception as exc:
             if solver := get_solver(exc):
                 return solver(request, exc)
@@ -250,6 +232,12 @@ class Endpoint[R]:
                     media_type="text/event-stream",
                     status_code=self._status_code,
                 )
+            elif self._static:
+                resp = StaticResponse(
+                    self._encoder(raw_return),
+                    media_type=self._media_type,
+                    status_code=self._status_code,
+                )
             else:
                 resp = Response(
                     content=self._encoder(raw_return),
@@ -266,7 +254,6 @@ class Endpoint[R]:
             return await response(scope, receive, send)
         if self._static:  # when there is no params at all
             raw_return = await self.make_static_call(scope, receive, send)
-            # response = StaticResponse(raw_return)
         else:
             raw_return = await self.make_call(scope, receive, send, self._graph)
         response = self.return_to_response(raw_return)
@@ -274,37 +261,18 @@ class Endpoint[R]:
 
 
 class RouteBase(ASGIBase):
-    _flyweights: dict[str, "Self"] = {}
-
-    def __new__(cls, path: str = "", **_) -> Self:
-        p = trim_path(path)
-        if p_route := cls._flyweights.get(p):
-            return p_route
-
-        cls._flyweights[p] = route = super().__new__(cls)
-        if parent := cls._flyweights.get(get_parent_path(p)):
-            if route not in parent.subroutes:
-                parent.subroutes.append(route)
-        return route
-
     def __init__(  # type: ignore
         self,
         path: str = "",
         *,
         graph: Graph | None = None,
-        registry: MessageRegistry | None = None,
-        listeners: list[Callable[..., Any]] | None = None,
         middlewares: list[MiddlewareFactory[Any]] | None = None,
     ):
         super().__init__(middlewares)
         self.path = trim_path(path)
-        self.path_regex: Pattern[str] | None = None
+        self.path_regex: Pattern[str] = build_path_regex(self.path)
         self.graph = graph or Graph(self_inject=False)
-        self.registry = registry or MessageRegistry(event_base=Event, graph=graph)
-        if listeners:
-            self.registry.register(*listeners)
-        self.busterm = BusTerminal(self.registry, graph=graph)
-        self.app_config: AppConfig | None = None
+        self.workers = None
         self.subroutes: list[Self] = []
 
     def __truediv__(self, path: str) -> "Self":
@@ -321,7 +289,7 @@ class RouteBase(ASGIBase):
         rest = self.path.removeprefix(other_path)
         return rest.count("/") < 2
 
-    def sub(self, path: str) -> "Self":
+    def sub(self, path: str) -> Self:
         sub_path = trim_path(path)
         merged_path = merge_path(self.path, sub_path)
         for sub in self.subroutes:
@@ -330,43 +298,33 @@ class RouteBase(ASGIBase):
         sub = self.__class__(
             path=merged_path,
             graph=self.graph,
-            registry=self.registry,
             middlewares=self.middle_factories,
         )
         self.subroutes.append(sub)
         return sub
 
-    def match(self, scope: IScope) -> IScope | None:
+    def match(self, scope: IScope) -> bool:
         path = scope["path"]
         if not self.path_regex or not (m := self.path_regex.match(path)):
-            return None
+            return False
         scope["path_params"] = m.groupdict()
-        return scope
+        return True
 
-    def add_nodes[T](
+    def add_nodes(
         self, *nodes: Union[IDependent[T], tuple[IDependent[T], INodeConfig]]
     ) -> None:
         self.graph.add_nodes(*nodes)
 
-    def factory[R](self, node: Callable[..., R], **node_config: Unpack[INodeConfig]):
+    def factory(self, node: Callable[..., R], **node_config: Unpack[INodeConfig]):
         return self.graph.node(node, **node_config)
 
-    def listen(self, listener: Callable[[Event, Any], None]) -> None:
-        self.registry.register(listener)
-
-    def has_listener(self, listener: Callable[..., Any]) -> bool:
-        event_metas = list(self.registry.event_mapping.values())
-
-        for metas in event_metas:
-            for meta in metas:
-                if meta.handler is listener:
-                    return True
-        return False
-
-    def setup(self, graph: Graph | None = None, busterm: BusTerminal | None = None):
+    async def setup(
+        self,
+        graph: Graph | None = None,
+        workers: ThreadPoolExecutor | None = None,
+    ):
         self.graph = graph or self.graph
-        self.busterm = busterm or self.busterm
-        self.app_config = lhl_get_config()
+        self.workers = workers
 
 
 class Route(RouteBase):
@@ -376,21 +334,22 @@ class Route(RouteBase):
         path: str = "",
         *,
         graph: Graph | None = None,
-        registry: MessageRegistry | None = None,
-        listeners: list[Callable[..., Any]] | None = None,
         middlewares: list[MiddlewareFactory[Any]] | None = None,
         props: EndpointProps | None = None,
     ):
         super().__init__(
             path,
             graph=graph,
-            registry=registry,
-            listeners=listeners,
             middlewares=middlewares,
         )
         self.endpoints: dict[HTTP_METHODS, Endpoint[Any]] = {}
         self.call_stacks: dict[HTTP_METHODS, ASGIApp] = {}
-        self.props = props or EndpointProps(tags=[generate_route_tag(self.path)])
+        if props is not None:
+            if props.tags is None:
+                props = props.replace(tags=generate_route_tag(self.path))
+            self.props = props
+        else:
+            self.props = EndpointProps(tags=[generate_route_tag(self.path)])
 
     def __repr__(self):
         endpoints_repr = "".join(
@@ -403,13 +362,22 @@ class Route(RouteBase):
         endpoint = self.call_stacks.get(scope["method"]) or METHOD_NOT_ALLOWED_RESP
         await endpoint(scope, receive, send)
 
-    def setup(self, graph: Graph | None = None, busterm: BusTerminal | None = None):
-        super().setup(graph, busterm)
+    async def setup(
+        self, graph: Graph | None = None, workers: ThreadPoolExecutor | None = None
+    ):
+        await super().setup(workers=workers, graph=graph)
+        self.endpoint_parser = EndpointParser(self.graph, self.path)
 
         for method, ep in self.endpoints.items():
-            ep.setup()
+            if ep.is_setup:
+                continue
+            ep_sig = self.endpoint_parser.parse(ep.unwrapped_func)
+            await ep.setup(ep_sig)
             self.call_stacks[method] = self.chainup_middlewares(ep)
 
+    def parse_endpoint(self, func: Callable[..., R]) -> EndpointSignature[R]:
+        return self.endpoint_parser.parse(func)
+
     def get_endpoint(
         self, method_func: HTTP_METHODS | Callable[..., Any]
     ) -> Endpoint[Any]:
@@ -423,11 +391,38 @@ class Route(RouteBase):
         else:
             raise KeyError(f"{method_func} is not in current route")
 
+    def include_subroutes(self, *subs: Self, parent_prefix: str | None = None) -> None:
+        """
+        Merge other routes into current route as sub routes,
+        a new route would be created based on the merged subroute
+
+        NOTE: This method is NOT idempotent
+        """
+        for sub in subs:
+            self.graph.merge(sub.graph)
+            if parent_prefix:
+                sub_path = sub.path.removeprefix(parent_prefix)
+            else:
+                sub_path = sub.path
+            merged_path = merge_path(self.path, sub_path)
+            sub_subs = sub.subroutes
+            new_sub = self.__class__(
+                path=merged_path,
+                graph=self.graph,
+                middlewares=sub.middle_factories,
+                props=sub.props,
+            )
+            for method, ep in sub.endpoints.items():
+                new_sub.add_endpoint(method, func=ep.unwrapped_func, **ep.props)
+            for sub_sub in sub_subs:
+                new_sub.include_subroutes(sub_sub, parent_prefix=sub.path)
+            self.subroutes.append(new_sub)
+
     def redirect(self, method: MethodType, **path_params: str) -> None:
         # owner = method.__self__
         raise NotImplementedError
 
-    def add_endpoint[**P, R](
+    def add_endpoint(
         self,
         *methods: HTTP_METHODS,
         func: Func[P, R],
@@ -442,32 +437,28 @@ class Route(RouteBase):
 
         for method in methods:
             endpoint = Endpoint(
-                self,
-                method=method,
-                func=func,
-                props=props,
+                self, method=method, func=func, props=props, workers=self.workers
             )
             self.endpoints[method] = endpoint
-            if self.path_regex is None:
-                self.path_regex = build_path_regex(self.path)
+
         return func
 
     # ============ Http Methods ================
 
     @overload
-    def get[**P, R](
+    def get(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def get[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def get(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def get[**P, R](
+    def get(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R] | Callable[[Func[P, R]], Func[P, R]]: ...
 
-    def get[**P, R](
+    def get(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R] | Callable[[Func[P, R]], Func[P, R]]:
         if func is None:
@@ -475,19 +466,19 @@ class Route(RouteBase):
         return self.add_endpoint("GET", func=func, **epconfig)
 
     @overload
-    def put[**P, R](
+    def put(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def put[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def put(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def put[**P, R](
+    def put(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def put[**P, R](
+    def put(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -495,19 +486,19 @@ class Route(RouteBase):
         return self.add_endpoint("PUT", func=func, **epconfig)
 
     @overload
-    def post[**P, R](
+    def post(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def post[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def post(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def post[**P, R](
+    def post(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def post[**P, R](
+    def post(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -515,19 +506,19 @@ class Route(RouteBase):
         return self.add_endpoint("POST", func=func, **epconfig)
 
     @overload
-    def delete[**P, R](
+    def delete(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def delete[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def delete(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def delete[**P, R](
+    def delete(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def delete[**P, R](
+    def delete(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -535,19 +526,19 @@ class Route(RouteBase):
         return self.add_endpoint("DELETE", func=func, **epconfig)
 
     @overload
-    def patch[**P, R](
+    def patch(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def patch[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def patch(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def patch[**P, R](
+    def patch(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def patch[**P, R](
+    def patch(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -555,19 +546,19 @@ class Route(RouteBase):
         return self.add_endpoint("PATCH", func=func, **epconfig)
 
     @overload
-    def head[**P, R](
+    def head(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def head[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def head(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def head[**P, R](
+    def head(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def head[**P, R](
+    def head(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -575,19 +566,19 @@ class Route(RouteBase):
         return self.add_endpoint("HEAD", func=func, **epconfig)
 
     @overload
-    def options[**P, R](
+    def options(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def options[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def options(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def options[**P, R](
+    def options(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def options[**P, R](
+    def options(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -595,19 +586,19 @@ class Route(RouteBase):
         return self.add_endpoint("OPTIONS", func=func, **epconfig)
 
     @overload
-    def trace[**P, R](
+    def trace(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def trace[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def trace(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def trace[**P, R](
+    def trace(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def trace[**P, R](
+    def trace(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
@@ -615,19 +606,19 @@ class Route(RouteBase):
         return self.add_endpoint("TRACE", func=func, **epconfig)
 
     @overload
-    def connect[**P, R](
+    def connect(
         self, **epconfig: Unpack[IEndpointProps]
     ) -> Callable[[Func[P, R]], Func[P, R]]: ...
 
     @overload
-    def connect[**P, R](self, func: Func[P, R]) -> Func[P, R]: ...
+    def connect(self, func: Func[P, R]) -> Func[P, R]: ...
 
     @overload
-    def connect[**P, R](
+    def connect(
         self, func: Func[P, R] | None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]: ...
 
-    def connect[**P, R](
+    def connect(
         self, func: Func[P, R] | None = None, **epconfig: Unpack[IEndpointProps]
     ) -> Func[P, R]:
         if func is None:
