@@ -1,11 +1,10 @@
 import re
 from re import Pattern
-from typing import Any, Literal, TypeVar, get_args, get_origin
 
+# from typing import Any, Literal, TypeVar, get_args, get_origin
+# from lihil.errors import NotSupportedError
 from starlette.routing import compile_path
 
-from lihil.errors import NotSupportedError
-
 RE_PATH_KEYS = re.compile(r"\{([a-zA-Z_][a-zA-Z0-9_]*)\}")
 "Must be a valid python variable name?"
 
@@ -19,7 +18,7 @@ def to_kebab_case(string: str) -> str:
     Examples:
         HTTPException -> http-exception
         UserAPI -> user-api
-        OAuth2PasswordBearer -> oauth2-password-bearer
+        OAuth2PasswordBearer -> o-auth2-password-bearer
     """
     if not string:
         return string
@@ -70,15 +69,15 @@ def trim_path(path: str) -> str:
     return path
 
 
-def get_parent_path(path: str) -> str:
-    # Remove trailing slash if present
-    if path.endswith("/"):
-        return path[:-1]
+# def get_parent_path(path: str) -> str:
+#     # Remove trailing slash if present
+#     if path.endswith("/"):
+#         return path[:-1]
 
-    # Find the last slash and return everything before it
-    last_slash_index = path.rindex("/")
-    parent = path[:last_slash_index]
-    return parent
+#     # Find the last slash and return everything before it
+#     last_slash_index = path.rindex("/")
+#     parent = path[:last_slash_index]
+#     return trim_path(parent)
 
 
 def merge_path(parent_path: str, sub_path: str) -> str:
@@ -130,22 +129,6 @@ def build_path_regex(path: str, path_params: None = None) -> Pattern[str]:
     return path_regex
 
 
-def parse_header_key(name: str, key: Any) -> str:
-    if key is None or isinstance(key, TypeVar): # default case
-        return to_kebab_case(name)
-
-    if isinstance(key, str):
-        return key
-    elif get_origin(key) is Literal:
-        key = get_args(key)[0]
-    else:
-        raise NotSupportedError(f"Invalid header key {key}")
-
-    if not isinstance(key, str):
-        raise NotSupportedError(f"Invalid header key {key}")
-    return key
-
-
 def trimdoc(doc: str | None):
     """
     remove empty characters before and after doc
