@@ -1,26 +1,22 @@
 import argparse
-from pathlib import Path
 from unittest.mock import patch
 
 import pytest
 
 from lihil import Lihil
-from lihil.config import AppConfig, AppConfiguringError
-from lihil.config.parser import (
-    StoreTrueIfProvided,
-    build_parser,
-    config_from_cli,
-    config_from_file,
-    format_nested_dict,
-)
+from lihil.config import AppConfig, ConfigBase, lhl_get_config, lhl_read_config
+from lihil.config.loader import ConfigLoader, convert, load_from_cli
+from lihil.config.parser import StoreTrueIfProvided, build_parser, format_nested_dict
 from lihil.interface import MISSING
 
 # from lihil.config import AppConfig
 
 
 def test_app_read_config():
-    lhl = Lihil[None](config_file="pyproject.toml")
-    assert lhl.app_config.oas.doc_path == "/docs"
+    config = lhl_read_config("settings.toml")
+    Lihil(app_config=config)
+    config = lhl_get_config()
+    assert config.oas.DOC_PATH == "/docs"
 
 
 def test_format_nested_dict():
@@ -75,37 +71,37 @@ def test_app_config_build_parser():
     actions = {action.dest: action for action in parser._actions}
 
     # Check top-level arguments
-    assert "version" in actions
-    assert "is_prod" in actions
+    assert "VERSION" in actions
+    assert "IS_PROD" in actions
 
     # Check nested arguments
-    assert "oas.title" in actions
-    assert "oas.doc_path" in actions
+    assert "oas.TITLE" in actions
+    assert "oas.DOC_PATH" in actions
 
 
-@patch("sys.argv", ["prog", "--version", "2.0.0", "--oas.title", "Custom API"])
+@patch("sys.argv", ["prog", "--VERSION", "2.0.0", "--oas.TITLE", "Custom API"])
 def test_config_from_cli():
     """Test that config_from_cli correctly parses command line arguments."""
-    config_dict = config_from_cli(config_type=AppConfig)
+    config_dict = load_from_cli(config_type=AppConfig)
 
     assert config_dict is not None
-    assert config_dict["version"] == "2.0.0"
-    assert config_dict["oas"]["title"] == "Custom API"
+    assert config_dict["VERSION"] == "2.0.0"
+    assert config_dict["oas"]["TITLE"] == "Custom API"
 
 
-@patch("sys.argv", ["prog", "--is_prod"])
+@patch("sys.argv", ["prog", "--IS_PROD"])
 def test_config_from_cli_boolean_flag():
     """Test that boolean flags are correctly handled."""
-    config_dict = config_from_cli(config_type=AppConfig)
+    config_dict = load_from_cli(config_type=AppConfig)
 
     assert config_dict is not None
-    assert config_dict["is_prod"] is True
+    assert config_dict["IS_PROD"] is True
 
 
 @patch("sys.argv", ["prog"])
 def test_config_from_cli_no_args():
     """Test that config_from_cli returns None when no arguments are provided."""
-    config_dict = config_from_cli(config_type=AppConfig)
+    config_dict = load_from_cli(config_type=AppConfig)
 
     assert config_dict is None
 
@@ -113,16 +109,35 @@ def test_config_from_cli_no_args():
 @patch("sys.argv", ["prog", "--unknown-arg", "value"])
 def test_config_from_cli_unknown_args():
     """Test that config_from_cli ignores unknown arguments."""
-    config_dict = config_from_cli(config_type=AppConfig)
+    config_dict = load_from_cli(config_type=AppConfig)
     assert config_dict is None  # No recognized arguments
 
 
-def test_app_config_from_filepath(tmp_path: Path):
-    toml_file = tmp_path / "config.toml"
+# def test_app_load_configpath(tmp_path: Path):
+#     toml_file = tmp_path / "config.toml"
 
-    toml_file.touch()
+#     toml_file.touch()
 
-    # data = {"name": "example", "version": "1.0.0"}
+#     with pytest.raises(AppConfiguringError):
+#         res = ConfigLoader().load_config(toml_file)
 
-    with pytest.raises(AppConfiguringError):
-        config_from_file(toml_file)
+
+def test_enhance_app_config():
+    class SupabaseConfig(ConfigBase):
+        url: str
+        key: str
+
+    class MyConfig(AppConfig, kw_only=True):
+        supabase: SupabaseConfig
+
+    res = load_from_cli(
+        ["prog", "--supabase.url", "myurl", "--supabase.key", "mykey"],
+        config_type=MyConfig,
+    )
+    config = convert(res, MyConfig)
+    assert config.supabase.url == "myurl"
+    assert config.supabase.key == "mykey"
+
+
+def test_loader():
+    loader = ConfigLoader()
