@@ -5,30 +5,27 @@ from typing import Optional, Union
 import pytest
 from msgspec.structs import FieldInfo
 
-from lihil.config import OASConfig, ServerConfig
-from lihil.config.parser import (
+from lihil import Request
+from lihil.config import (
+    DEFAULT_CONFIG,
     AppConfig,
+    ConfigLoader,
+    lhl_get_config,
+    lhl_set_config,
+)
+from lihil.config.loader import deep_update, load_from_cli
+from lihil.config.parser import (
     ConfigBase,
     StoreTrueIfProvided,
     build_parser,
-    config_from_cli,
-    config_from_file,
-    deep_update,
     format_nested_dict,
     generate_parser_actions,
-    get_thread_cnt,
-    is_provided,
     parse_field_type,
 )
 from lihil.errors import AppConfiguringError
-from lihil.interface import MISSING, Maybe
-
-
-def test_get_thread_cnt():
-    """Test that get_thread_cnt returns a positive integer"""
-    thread_count = get_thread_cnt()
-    assert isinstance(thread_count, int)
-    assert thread_count > 0
+from lihil.interface import MISSING, Maybe, is_provided
+from lihil.plugins.bus import EventBus
+from lihil.signature.parser import is_lhl_primitive
 
 
 def test_format_nested_dict():
@@ -81,13 +78,13 @@ def test_store_true_if_provided():
     assert getattr(args, "flag_provided", False) is False
 
 
-# def test_is_lhl_dep():
-#     """Test identification of lihil dependencies"""
-#     assert is_lhl_dep(Request) is True
-#     assert is_lhl_dep(EventBus) is True
-#     assert is_lhl_dep(str) is False
-#     assert is_lhl_dep(int) is False
-#     assert is_lhl_dep(AppConfig) is False
+def test_is_lhl_dep():
+    """Test identification of lihil dependencies"""
+    assert is_lhl_primitive(Request) is True
+    assert is_lhl_primitive(EventBus) is False
+    assert is_lhl_primitive(str) is False
+    assert is_lhl_primitive(int) is False
+    assert is_lhl_primitive(AppConfig) is False
 
 
 def test_parse_field_type():
@@ -125,80 +122,79 @@ def test_build_parser():
     # Check that basic arguments are added
     actions = {action.dest: action for action in parser._actions}
 
-    assert "is_prod" in actions
-    assert "version" in actions
-    assert "max_thread_workers" in actions
+    assert "IS_PROD" in actions
+    assert "VERSION" in actions
 
     # Check nested config arguments
-    assert "oas.title" in actions
-    assert "server.host" in actions
-    assert "server.port" in actions
+    assert "oas.TITLE" in actions
+    assert "server.HOST" in actions
+    assert "server.PORT" in actions
 
 
-def test_config_from_file_toml():
+def test_load_config_toml():
     """Test loading config from a TOML file"""
     with tempfile.NamedTemporaryFile(suffix=".toml", mode="w+") as tmp:
         tmp.write(
             """
-        [tool.lihil]
-        is_prod = true
-        version = "1.0.0"
+        [lihil]
+        IS_PROD = true
+        VERSION = "1.0.0"
 
-        [tool.lihil.oas]
-        title = "Test API"
+        [lihil.oas]
+        TITLE = "Test API"
 
-        [tool.lihil.server]
-        host = "127.0.0.1"
-        port = 9000
+        [lihil.server]
+        HOST = "127.0.0.1"
+        PORT = 9000
         """
         )
         tmp.flush()
 
-        config = config_from_file(tmp.name)
+        config = ConfigLoader().load_config(tmp.name)
 
-        assert config.is_prod is True
-        assert config.version == "1.0.0"
-        assert config.oas.title == "Test API"
-        assert config.server.host == "127.0.0.1"
-        assert config.server.port == 9000
+        assert config.IS_PROD is True
+        assert config.VERSION == "1.0.0"
+        assert config.oas.TITLE == "Test API"
+        assert config.server.HOST == "127.0.0.1"
+        assert config.server.PORT == 9000
 
 
-def test_config_from_file_toml_alternative_format():
+def test_load_config_toml_alternative_format():
     """Test loading config from a TOML file with alternative format"""
     with tempfile.NamedTemporaryFile(suffix=".toml", mode="w+") as tmp:
         tmp.write(
             """
         [lihil]
-        is_prod = true
-        version = "1.0.0"
+        IS_PROD = true
+        VERSION = "1.0.0"
 
         [lihil.oas]
-        title = "Test API"
+        TITLE = "Test API"
         """
         )
         tmp.flush()
 
-        config = config_from_file(tmp.name)
+        config = ConfigLoader().load_config(tmp.name)
 
-        assert config.is_prod is True
-        assert config.version == "1.0.0"
-        assert config.oas.title == "Test API"
+        assert config.IS_PROD is True
+        assert config.VERSION == "1.0.0"
+        assert config.oas.TITLE == "Test API"
 
 
-def test_config_from_file_nonexistent():
+def test_load_config_nonexistent():
     """Test error when config file doesn't exist"""
     with pytest.raises(AppConfiguringError, match="not exist"):
-        config_from_file("nonexistent_file.toml")
+        ConfigLoader().load_config("nonexistent_file.toml")
 
 
-def test_config_from_file_unsupported_format():
+def test_load_config_unsupported_format():
     """Test error when config file has unsupported format"""
     with tempfile.NamedTemporaryFile(suffix=".xyz") as tmp:
-        with pytest.raises(AppConfiguringError, match="Not supported file type"):
-            config_from_file(tmp.name)
+        with pytest.raises(AppConfiguringError, match="Loader for .xyz is not found"):
+            ConfigLoader().load_config(tmp.name)
 
 
-def test_config_from_file_missing_table():
+def test_load_config_missing_table():
     """Test error when TOML file doesn't have lihil table"""
     with tempfile.NamedTemporaryFile(suffix=".toml", mode="w+") as tmp:
         tmp.write(
@@ -209,33 +205,22 @@ def test_config_from_file_missing_table():
         )
         tmp.flush()
 
-        with pytest.raises(AppConfiguringError, match="can't find table lihil"):
-            config_from_file(tmp.name)
-
-
-def test_config_from_file_default():
-    """Test default config when no file is provided"""
-    config = config_from_file(None)
-
-    assert isinstance(config, AppConfig)
-    assert config.is_prod is False
-    assert config.version == "0.1.0"
-    assert isinstance(config.oas, OASConfig)
-    assert isinstance(config.server, ServerConfig)
+        with pytest.raises(AppConfiguringError):
+            ConfigLoader().load_config(tmp.name)
 
 
 def test_config_from_cli(monkeypatch):
     """Test loading config from command line arguments"""
     # Mock sys.argv to simulate command line arguments
-    test_args = ["prog", "--is_prod", "--version", "2.0.0", "--server.port", "8080"]
+    test_args = ["prog", "--IS_PROD", "--VERSION", "2.0.0", "--server.PORT", "8080"]
     monkeypatch.setattr("sys.argv", test_args)
 
-    config_dict = config_from_cli(AppConfig)
+    config_dict = load_from_cli(config_type=AppConfig)
 
     assert config_dict is not None
-    assert config_dict["is_prod"] is True
-    assert config_dict["version"] == "2.0.0"
-    assert config_dict["server"]["port"] == 8080
+    assert config_dict["IS_PROD"] is True
+    assert config_dict["VERSION"] == "2.0.0"
+    assert config_dict["server"]["PORT"] == 8080
 
 
 def test_config_from_cli_empty(monkeypatch):
@@ -244,7 +229,7 @@ def test_config_from_cli_empty(monkeypatch):
     test_args = ["prog"]
     monkeypatch.setattr("sys.argv", test_args)
 
-    config_dict = config_from_cli(AppConfig)
+    config_dict = load_from_cli(config_type=AppConfig)
 
     assert config_dict is None
 
@@ -252,11 +237,11 @@ def test_config_from_cli_empty(monkeypatch):
 def test_config_from_cli_should_filter_provided_flags(monkeypatch):
     """Test that config_from_cli should filter out _provided attributes"""
     # Mock sys.argv with a boolean flag
-    test_args = ["prog", "--is_prod"]
+    test_args = ["prog", "--IS_PROD"]
     monkeypatch.setattr("sys.argv", test_args)
 
     # Get CLI config
-    cli_config = config_from_cli(AppConfig)
+    cli_config = load_from_cli(config_type=AppConfig)
 
     # The current implementation includes _provided flags, which is problematic
     assert cli_config is not None
@@ -267,8 +252,8 @@ def test_config_from_cli_should_filter_provided_flags(monkeypatch):
         assert not key.endswith("_provided"), f"Found unexpected _provided flag: {key}"
 
     # Check that the actual flag is still there
-    assert "is_prod" in cli_config
-    assert cli_config["is_prod"] is True
+    assert "IS_PROD" in cli_config
+    assert cli_config["IS_PROD"] is True
 
 
 def test_config_from_cli_fix():
@@ -309,13 +294,24 @@ def test_filed_type():
 def test_build_parser_with_bool():
 
     class NestedConfig(ConfigBase):
-        is_prod: bool
+        name: str
 
     class NewConfig(ConfigBase):
         nested: NestedConfig
 
     parser = build_parser(NewConfig)
+    res = parser.parse_args(["--nested.name", "lihil"])
+    assert getattr(res, "nested.name") == "lihil"
 
 
 def test_generate_app_confg_acotions():
     generate_parser_actions(AppConfig)
+
+
+def test_empty_set_config_reset_config():
+    config = AppConfig(IS_PROD=True)
+    lhl_set_config(config)
+    assert lhl_get_config() is config
+
+    lhl_set_config()
+    assert lhl_get_config() is DEFAULT_CONFIG
